 (**)
 (*  Programm : EXAMPLE 001                                                *)
 (*             Beispielprogramm fr Units ONEFILE1 und KEYFILE1 sowie     *)
 (*             UWEERR01 und UWEUNI01.                                     *)
 (* ---------------------------------------------------------------------- *)
 (*     Datei : EXAMP001.PAS                                               *)
 (*             1. Beispieldatei.                                          *)
 (* ---------------------------------------------------------------------- *)
 (*   Version : 0.1                                                        *)
 (*   History : 13.04.1995 Uwe Keim - Anfang.                              *)
 (**)

 PROGRAM Examp001;

  (* COMPILER-EINSTELLUNGEN: *)

  {$A-,B-,E+,F+,G+,I-,N+,O-,P-,Q-,R-,S-,T-,V-,X+}
  {$M 16384,0,655360}

  {$UNDEF  DEBUG}                       (* Debug-Infos AN/AUS. *)

  {$IFDEF DEBUG}
  {$D+,L+,Y+}                           (* Debug-Infos. *)
  {$ELSE}
  {$D-,L-,Y-}                           (* Debug-Infos. *)
  {$ENDIF}

  (* DEKLARATIONSTEIL: *)

   USES Crt,

        OneFile1,
        KeyFile1,

        UweERR01,
        UweUNI01;

   (*  Verschiedene Erklrungen:  *)

   (* ----------------------------------------------------------- *)
   (*  Dieses Programm dient als einfaches (!) Beispiel fr       *)
   (*  einige Routinen der Units                                  *)
   (*                                                             *)
   (*   ONEFILE1,                                                 *)
   (*   KEYFILE1,                                                 *)
   (*   UWEERR01,                                                 *)
   (*   UWEUNI01.                                                 *)
   (*                                                             *)
   (*  Da die Quelltexte dieser Units vorliegen, sollte dieses    *)
   (*  Beispiel gengen !                                         *)
   (*                                                             *)
   (*  Hinweis: Wenn eine OneFile-Datei mit KeyFile verschlsselt *)
   (*           ist, mu dies mit dem Schalter {$DEFINE KEYFILE}  *)
   (*           in der Unit ONEFILE1 angegeben werden !!          *)
   (* ----------------------------------------------------------- *)

   (*  Verschiedene Typen, Konstanten und Variablen:  *)

         (* Eigene Fehlercodes: z.B. 8000-9000: *)
   CONST EXP_Error_Test01                = 8000;
         EXP_Error_Test02                = 8001;
         EXP_Error_Test03                = 8002;
         { .. usw. }

   (*  Prototypen der Prozeduren:  *)

   (*  NORMALE PROZEDUREN:  *)

   { Keine. }

   (*  SYSTEM-ROUTINEN:  *)

   FUNCTION EXPGetErrorMessage(AS : String) : String; Forward;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Liefert den Text zum aktuellen Fehler in ERR_ERROR.  *)
    (*  Eingabe  : ERR_ERROR = Fehlernummer,                            *)
    (*             AS        = Eventueller zustzlicher Text (Z.B.      *)
    (*                         Dateiname bei Fehler 'Nicht gefunden').  *)
    (*  Rckgabe : Fehlertext.                                          *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE Init; Forward;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Initialisiert das Programm.                          *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE Main; Forward;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Hauptteil des Programms, ruft alles auf.             *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE Done; Forward;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Beendet das Programm.                                *)
    (* ---------------------------------------------------------------- *)

   (*  Prozedurrmpfe:  *)

   (*  *)
   (*  NORMALE PROZEDUREN:  *)
   (*  *)

   { Keine. }

   (*  *)
   (*  SYSTEM-ROUTINEN:  *)
   (*  *)

   FUNCTION EXPGetErrorMessage(AS : String) : String;
    (* ---------------------------------------------------------------- *)
    Var S : String;
        C : String;
    Begin
     If AS <> '' Then C := ' ' Else C := '';

     Str(ERR_Error, S);
     S := 'Fehler Nr. ' + S + ' : ';

     Case ERR_Error Of
      (* Eigene Fehlercodes: *)
      EXP_Error_Test01                : S := S + 'Test-Fehler 01 erzeugt, kein String bergeben.';
      EXP_Error_Test02                : S := S + 'Test-Fehler 02 erzeugt, String ' + UpString(AS + C) + 'bergeben.';
      EXP_Error_Test03                : S := S + 'Test-Fehler 03 erzeugt, String ' + UpString(AS + C) + 'bergeben.';

      Else                              S := '';
     End;

     EXPGetErrorMessage := S;
    End;

   PROCEDURE Init;
    (* ---------------------------------------------------------------- *)
    Begin
     (* Eigene Close-Prozedur einbinden: *)
     InsertErrorProc(Done);

     (* Eigene Messages einbinden: *)
     InsertErrorMessageFunc(EXPGetErrorMessage);
    End;

   PROCEDURE Main;
    (* ---------------------------------------------------------------- *)
    Var PH1 : Pointer;
        BH1 : Byte;
    Begin
     (* -- 1. ONEFILE-Lesen: ----------------------------------------- *)

     (* Schirm lschen: *)
     ClrScr;

     (* HAUPT-Datei ffnen: *)
     AssignMAIN(F, 'EXAMP001.DAT');
     ResetMAIN(F);

     (* UNTER-Datei in HAUPT-Datei ffnen: *)
     _Assign(F, 'SUB01.TXT');
     _Reset(F, 1);                      (* BlockRead-Lnge : 1 Byte. *)

     (* Lese bis Ende der UNTER-Datei: *)
     While Not _EOF(F) Do
      Begin
       PH1 := @BH1;
       _BlockRead(F, PH1, SizeOf(BH1)); (* Lese 1 Byte. *)
       Writeln(BH1);                    (* Auf Schirm ausgeben. *)
      End;

     (* Schliee UNTER-Datei: *)
     _Close(F);

     (* Schliee HAUPT-Datei: *)
     CloseMAIN(F);

     (* Warte auf Taste: *)
     Write(#13#10#13#10'<Taste!>');
     While KeyPressed Do ReadKey; Repeat Until KeyPressed;


     (* -- 2. ERROR erzeugen: ---------------------------------------- *)

     {$DEFINE DoError02}                (* Welcher Fehler erzeugt wird. *)

     {$IFDEF DoError01}
     ERR_Error := EXP_Error_Test01;     (* Globale Fehlervariable setzen. *)
     CriticalError('');                 (* Fehler-Routine aufrufen. *)
     {$ENDIF}

     {$IFDEF DoError02}
     ERR_Error := EXP_Error_Test03;     (* Globale Fehlervariable setzen. *)
     CriticalError('bergabestring');   (* Fehler-Routine aufrufen. *)
     {$ENDIF}

     {$IFDEF DoError03}
     ERR_Error := EXP_Error_Test03;     (* Globale Fehlervariable setzen. *)
     CriticalError('Noch''n String');   (* Fehler-Routine aufrufen. *)
     {$ENDIF}

     (* Warte auf Taste: *)
     Write(#13#10#13#10'<Taste!>');
     While KeyPressed Do ReadKey; Repeat Until KeyPressed;
    End;

   PROCEDURE Done;
    (* ---------------------------------------------------------------- *)
    Begin
    End;

  (*  HAUPTPROGRAMM:  *)

   BEGIN
    Init;
    Main;
    Done;
   END.

 (**)