 (**)
 (*  Programm : UWEUNI01                                                   *)
 (*             Allgemeine (UNIversal) Routinen.                           *)
 (* ---------------------------------------------------------------------- *)
 (*     Datei : UWEUNI01.PAS                                               *)
 (*             Die Unit.                                                  *)
 (* ---------------------------------------------------------------------- *)
 (*   Version : 0.1                                                        *)
 (*   History : 31.08.1994 Uwe Keim - Anfang.                              *)
 (**)

 UNIT UWEUNI01;

  {$A-,B-,D-,E+,F+,G+,I-,L-,N+,O-,P-,Q-,R-,S-,T-,V-,X+,Y-}
  {$M 16384,0,655360}

  {$undef  DEBUG}                       (* Debug-Infos AN/AUS. *)

  {$IFDEF DEBUG}
  {$D+,L+,Y+}                           (* Debug-Infos. *)
  {$ENDIF}

  (* -- FFENTLICHER TEIL: ---------------------------------------------- *)
  INTERFACE

   USES Dos,
        UweERR01;

   (*  Verschiedene Erklrungen:  *)

   (* ----------------------------------------------------------- *)
   (*  Enthlt Funktionen aus "MEMORY.PAS" um an Segment-Anfang   *)
   (*  Speicher zu reservieren (Offset garantiert 0).             *)
   (* ----------------------------------------------------------- *)

   (*  Verschiedene Typen:  *)

   TYPE  TProc   = String[20];         (* Prozessorname. *)

         PtrRec = Record
                   Ofs, Seg : Word;
                  End;

   (*  Verschiedene Konstanten:  *)

   CONST (* Fehler-Codes: *)

         (* Eigene Fehlercodes: 0500-0549: *)
         UNI_Error_AccessKeyBController  = 0500;

   (*  Verschiedene Variablen:  *)

   VAR OldDelay  : Byte;                (* Start-Tastaturrate. *)
       OldSpeed  : Byte;                (* Start-Tastaturrate. *)
       Regs      : Registers;           (* Die Register. *)

       (* BIOS-Variablen: *)
       CharHeight : Word    Absolute $0000:$0485;(* BIOS: Zeichenhhe. *)
       FontPtr    : Pointer Absolute $0000:$010C;(* INT 43h-Vektor. *)

   (*  Prototypen der Prozeduren:  *)

   PROCEDURE UNIInitRoutines;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Initialisiert alles Ntige.                          *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE UNICloseRoutines;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Beendet alles ordnungsgem.                         *)
    (* ---------------------------------------------------------------- *)

   FUNCTION UNIGetErrorMessage(AS : String) : String;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Liefert den Text zum aktuellen Fehler in ERR_Error.  *)
    (*  Eingabe  : ERR_Error = Fehlernummer,                            *)
    (*             AS        = Eventueller zustzlicher Text (Z.B.      *)
    (*                         Dateiname bei Fehler 'Nicht gefunden').  *)
    (*  Rckgabe : Fehlertext.                                          *)
    (* ---------------------------------------------------------------- *)

   FUNCTION DirString(S : PathStr) : DirStr;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Liefert den DIR-Anteil eines Strings.                *)
    (*  Eingabe  : S = Kompletter String.                               *)
    (*  Rckgabe : Der Dir-Anteil.                                      *)
    (* ---------------------------------------------------------------- *)

   FUNCTION NameString(S : PathStr) : NameStr;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Liefert den NAMNES-Anteil eines Strings.             *)
    (*  Eingabe  : S = Kompletter String.                               *)
    (*  Rckgabe : Der Name-Anteil.                                     *)
    (* ---------------------------------------------------------------- *)

   FUNCTION ExtString(S : PathStr) : ExtStr;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Liefert den ERWEITERUNGS-Anteil eines Strings.       *)
    (*  Eingabe  : S = Kompletter String.                               *)
    (*  Rckgabe : Der Erweiterungs-Anteil.                             *)
    (* ---------------------------------------------------------------- *)

   FUNCTION UpString(S : String) : String;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Wandelt alle Gro- in Kleinbuchstaben.               *)
    (*  Eingabe  : S = Der zu wandelnde String.                         *)
    (*  Rckgabe : Der gewandelte String.                               *)
    (* ---------------------------------------------------------------- *)

   FUNCTION FileExists(FileName : String) : Boolean;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Testet, ob eine Datei vorhanden ist.                 *)
    (*  Eingabe  : FILENAME = Dateiname.                                *)
    (*  Rckgabe : TRUE, wenn vorhanden, sonst FALSE.                   *)
    (*  Hinweis  : Falls die Datei offen ist, wird sie geschlossen.     *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE SetTypmRate(ADelay, ASpeed : Byte);
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Setzt die Typmatic-Rate der Tastatur.                 *)
    (* Eingabe  : ADELAY = Verzgerung, bis Wiederholung einsetzt,      *)
    (*            ASPEED = Wiederholungsrate.                           *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE GetTypmRate(Var ADelay : Byte; Var ASpeed : Byte);
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Liest die Typmatic-Rate der Tastatur.                 *)
    (* Rckgabe : ADELAY = Verzgerung, bis Wiederholung einsetzt,      *)
    (*            ASPEED = Wiederholungsrate.                           *)
    (* ---------------------------------------------------------------- *)

   FUNCTION AT : Boolean;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Testen, ob mindestens ein AT ( >= 80286 ).            *)
    (* Rckgabe : TRUE, wenn mindestens ein AT, sonst FALSE.            *)
    (* ---------------------------------------------------------------- *)

   FUNCTION Is386 : Boolean;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Testen, ob mind. ein 80386 Prozessor vorhanden ist.   *)
    (* Rckgabe : TRUE, falls ja, sonst FALSE;                          *)
    (* ---------------------------------------------------------------- *)

   FUNCTION GetProzessor : TProc;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Ermittelt den Prozessor im Rechner.                   *)
    (* Rckgabe : Name des Prozessors (siehe "PROCS").                  *)
    (* ---------------------------------------------------------------- *)

   FUNCTION GetCoProzessor : TProc;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Ermittelt den Co-Prozessor im Rechner.                *)
    (* Rckgabe : Name des Co-Prozessors (siehe "COPROCS").             *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE SetBorderColor(AColor : Byte);
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Frbt den Bildschirmrand.                            *)
    (*  Eingabe  : ACOLOR = Die zu setzende Farbe.                      *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE CursorAn;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Schaltet den Text-Cursor an.                         *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE CursorAus;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Schaltet den Text-Cursor aus.                        *)
    (* ---------------------------------------------------------------- *)

   FUNCTION MemAllocSeg(ASize: Word) : Pointer;
    (* ---------------------------------------------------------------- *)
    (*  Aufgabe  : Reserviert einen an einer Segment-Grenze beginnenden *)
    (*             Speicherbereich, der garantiert an Offset 0 beginnt. *)
    (*  Eingabe  : ASIZE = Gre des zu belegenden Speichers, maximal   *)
    (*                     64k - $08.                                   *)
    (*  Rckgabe : Zeiger auf Speicherbereich, wenn nicht mehr gengend *)
    (*             Speicher zur Verfgung, NIL.                         *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE WaitVRetrace;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Wartet auf den Beginn der vertikalen Retrace-Periode. *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE ScreenOff;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Schaltet den Bildschirm aus.                          *)
    (* ---------------------------------------------------------------- *)

   PROCEDURE ScreenOn;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Schaltet den Bildschirm ein.                          *)
    (* ---------------------------------------------------------------- *)

  (* -- PRIVATER TEIL: -------------------------------------------------- *)
  IMPLEMENTATION

   USES Crt;

   {$L UweUNI1a.obj}                    (* Testprozeduren fr Prozessoren. *)

   (*  Verschiedene Konstanten:  *)

   CONST Procs   : Array[0..8] Of TProc =
                    ( 'INTEL 8088',             (* Code 0. *)
                      'INTEL 8086',             (* Code 1. *)
                      'NEC V20',                (* Code 2. *)
                      'NEC V30',                (* Code 3. *)
                      'INTEL 80188',            (* Code 4. *)
                      'INTEL 80186',            (* Code 5. *)
                      'INTEL 80286',            (* Code 6. *)
                      'INTEL 80386',            (* Code 7. *)
                      'INTEL 80486' );          (* Code 8. *)

         CoProcs : Array[0..3] Of TProc =
                    ( 'Kein Co-Prozessor',      (* Code 0. *)
                      '8087',                   (* Code 1. *)
                      '80287',                  (* Code 2. *)
                      '80387/80487' );          (* Code 3. *)

   (*  Verschiedene Variablen:  *)

   (*  Prototypen der ImplementationsProzeduren:  *)

   FUNCTION GetProz : Integer; External;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Ermittelt den Prozessor im Rechner.                   *)
    (* Rckgabe : Nummer zwischen 0 und 8, die den Prozessor            *)
    (*            spezifiziert (siehe "PROCS").                         *)
    (* ---------------------------------------------------------------- *)

   FUNCTION GetCo : Integer; External;
    (* ---------------------------------------------------------------- *)
    (* Aufgabe  : Ermittelt den Co-Prozessor im Rechner.                *)
    (* Rckgabe : Nummer zwischen 0 und 3, die den Co-Prozessor         *)
    (*            spezifiziert (siehe "COPROCS").                       *)
    (* ---------------------------------------------------------------- *)

   (*  Rmpfe der ImplementationsProzeduren:  *)

   (*  InterfaceProzeduren:  *)

   PROCEDURE UNIInitRoutines;
    (* ---------------------------------------------------------------- *)
    Begin
     NoSound;

     (* Alte Tastaturrate auslesen: *)
     GetTypmRate(OldDelay, OldSpeed);

     (* Eigene Close-Prozedur einbinden: *)
     InsertErrorProc(UNICloseRoutines);

     (* Eigene Messages einbinden: *)
     InsertErrorMessageFunc(UNIGetErrorMessage);
    End;

   PROCEDURE UNICloseRoutines;
    (* ---------------------------------------------------------------- *)
    Begin
     (* Alte Tastaturrate setzen: *)
     SetTypmRate(OldDelay, OldSpeed);
    End;

   FUNCTION UNIGetErrorMessage(AS : String) : String;
    (* ---------------------------------------------------------------- *)
    Var S : String;
        C : String;
    Begin
     If AS <> '' Then C := ' ' Else C := '';

     Str(ERR_Error, S);
     S := 'Fehler Nr. ' + S + ' : ';

     Case ERR_Error Of
      (* Eigene Fehlercodes: *)
      UNI_Error_AccessKeyBController  : S := S + 'Fehler beim Zugriff auf Tastaturkontroller.';

      Else                              S := '';
     End;

     UNIGetErrorMessage := S;
    End;

   FUNCTION DirString(S : PathStr) : DirStr;
    (* ---------------------------------------------------------------- *)
    Var D : DirStr;
        N : NameStr;
        E : ExtStr;
    Begin
     FSplit(S, D, N, E);
     DirString := D;
    End;

   FUNCTION NameString(S : PathStr) : NameStr;
    (* ---------------------------------------------------------------- *)
    Var D : DirStr;
        N : NameStr;
        E : ExtStr;
    Begin
     FSplit(S, D, N, E);
     NameString := N;
    End;

   FUNCTION ExtString(S : PathStr) : ExtStr;
    (* ---------------------------------------------------------------- *)
    Var D : DirStr;
        N : NameStr;
        E : ExtStr;
    Begin
     FSplit(S, D, N, E);
     ExtString := E;
    End;

   FUNCTION UpString(S : String) : String;
    (* ---------------------------------------------------------------- *)
    Var I : Integer;
        H : String;
    Begin
     H := S;
     For I := 1 To Length(H) Do H[I] := UpCase(H[I]);
     UpString := H;
    End;

   FUNCTION FileExists(FileName : String) : Boolean;
    (* ---------------------------------------------------------------- *)
    Var F : File;
    Begin
     {$I-}
     Assign(F, FileName);
     Reset(F);
     Close(F);
     {$I+}
     FileExists := (IOResult = 0) And (FileName <> '');
    End;

   PROCEDURE SetTypmRate(ADelay, ASpeed : Byte);
    (* ---------------------------------------------------------------- *)
    Begin
     If AT Then                         (* BIOS-Funktion erst ab AT. *)
      Begin
       (* ber BIOS ansprechen: *)
       Asm
        mov   ax, 0305h
        mov   bh, ADelay
        mov   bl, ASpeed
        int   16h
       End;
      End;

     ERR_Error := ERR_Error_OK;
    End;

   PROCEDURE GetTypmRate(Var ADelay : Byte; Var ASpeed : Byte);
    (* ---------------------------------------------------------------- *)
    Var Delay, Speed : Integer;
    Begin
     Delay := 1;                        (* Standart bei AT: 500 ms. *)
     Speed := 12;                       (* Standart bei AT: 10 Zeichen/Sek. *)

     (* Rckgabe: *)
     ADelay := Delay;
     ASpeed := Speed;
    End;

   FUNCTION AT : Boolean;
    (* ---------------------------------------------------------------- *)
    Begin
     AT := Mem[$F000:$FFFE] = $FC;      (* Kennung fr >= AT. *)
    End;

   FUNCTION Is386 : Boolean;
    (* ---------------------------------------------------------------- *)
    Var IH1 : Integer;
    Begin
     IH1 := GetProz;                    (* Ermittle ProzessorNUMMER. *)
     Is386 := IH1 In [7,8];             (* Wert zurckgeben. *)
    End;

   FUNCTION GetProzessor : TProc;
    (* ---------------------------------------------------------------- *)
    Begin
     GetProzessor := Procs[GetProz];
    End;

   FUNCTION GetCoProzessor : TProc;
    (* ---------------------------------------------------------------- *)
    Begin
     GetCoProzessor := CoProcs[GetCo];
    End;

   PROCEDURE SetBorderColor(AColor : Byte); Assembler;
    (* ---------------------------------------------------------------- *)
    Asm
     mov   ax, 1001h
     mov   bh, AColor
     int   10h
    End;

   PROCEDURE CursorAn;
    (* ---------------------------------------------------------------- *)
    Begin
     Port[$3D4] := $A;
     Port[$3D5] := Port[$3D5] And $DF;
    End;

   PROCEDURE CursorAus;
    (* ---------------------------------------------------------------- *)
    Begin
     Port[$3D4] := $A;
     Port[$3D5] := Port[$3D5] Or $20;
    End;

   FUNCTION MemAllocSeg(ASize: Word) : Pointer;
    (* ---------------------------------------------------------------- *)
    Var P, T : Pointer;

    Function LowMemory : Boolean; Assembler;
     (* --------------------------------------------------------------- *)
     Asm
      MOV	AX, HeapEnd.Word[2]
      SUB	AX, HeapPtr.Word[2]
      SUB	AX, 256                 (* LowMemSize. *)
      SBB	AX, AX
      NEG	AX
     End;

    Function MemAlloc(ASize: Word) : Pointer;
     (* --------------------------------------------------------------- *)
     Var P : Pointer;
     Begin
      GetMem(P, ASize);
{     If (P <> NIL) And LowMemory Then
       Begin
        FreeMem(P, ASize);
        P := NIL;
       End;
}     MemAlloc := P;
     End;

    (* ---------------------------------------------------------------- *)
    Begin
     ASize := (ASize + 7) And $FFF8;
     P := MemAlloc(ASize + 8);
     If P <> NIL Then
      Begin
       If PtrRec(P).Ofs = 0 Then
        Begin
         PtrRec(T).Ofs := ASize And 15;
         PtrRec(T).Seg := PtrRec(P).Seg + ASize Shr 4;
        End
       Else
        Begin
         T := P;
         PtrRec(P).Ofs := 0;
         Inc(PtrRec(P).Seg);
        End;
       FreeMem(T, 8);
      End;
     MemAllocSeg := P;
    End;

   PROCEDURE WaitVRetrace; Assembler;
    (* ---------------------------------------------------------------- *)
    Asm
     mov   dx, 03DAh

    @WaitNotVSyncLoop:                  (* Falls noch aktiv, warte bis Ende. *)
     in    al, dx
     and   al, 8
     jnz   @WaitNotVSyncLoop

    @WaitVSyncLoop:                     (* Warte bis Beginn. *)
     in    al, dx
     and   al, 8
     jz    @WaitVSyncLoop
    End;

   PROCEDURE ScreenOff;
    (* ---------------------------------------------------------------- *)
    Begin
     Port[$3C4] := 1;
     Port[$3C5] := Port[$3C5] Or $20;
    End;

   PROCEDURE ScreenOn;
    (* ---------------------------------------------------------------- *)
    Begin
     Port[$3C4] := 1;
     Port[$3C5] := Port[$3C5] And $DF;
    End;

  (*  INITIALISIERUNGSTEIL:  *)

   BEGIN
    UNIInitRoutines;
   END.

 (**)